import { WBTNode } from 'wbtree';
export declare type SATNode<T> = WBTNode<{
    value: T;
    valueCount: number;
}>;
export declare const balanceLeft: (node: WBTNode<{
    valueCount: number;
}> | undefined) => WBTNode<{
    valueCount: number;
}> | undefined, balanceRight: (node: WBTNode<{
    valueCount: number;
}> | undefined) => WBTNode<{
    valueCount: number;
}> | undefined;
/**
 * Finds a node in the subtree rooted at a given node. It does this by calling a
 * special compare function that takes the current node as input and continues
 * depending on the output of this function:
 *
 * If the compare function returns a negative number on a given node, the search
 * will continue on the left subtree of the node.
 *
 * If the compare function returns a positive number on a given node, the search
 * will continue on the right subtree of the node.
 *
 * If the compare function returns zero on the node, then the current node will
 * be returned by `find`.
 *
 * @example
 * ```ts
 * // Assumes we have some compare function for the type of `value`.
 * const findByValue = (value, root) => find(root, (node) => {
 *   return compare(value, node.data.value)
 * })
 * ```
 *
 * @param node The root node of the tree to search.
 * @param progressUpdater The special compare function.
 */
export declare function find<T>(node: SATNode<T> | undefined, progressUpdater: (node: SATNode<T>) => number): SATNode<T> | undefined;
/**
 * Inserts the value into the tree. The value must not be in the tree already.
 */
export declare function insert<T>(value: T, node: SATNode<T> | undefined, compare: (a: T, b: T) => number): SATNode<T>;
/**
 * Removes a value from the subtree rooted at `node` and returns the root of the
 * resulting tree after deletion. The node must be in the tree already.
 *
 * @param value The value to remove.
 * @param node The root of the tree to remove the value from.
 * @param compare A comparison function to know which subtree to recurse into.
 */
export declare function remove<T>(value: T, node: SATNode<T>, compare: (a: T, b: T) => number): SATNode<T> | undefined;
